/* jshint node: true */
'use strict';

var Funnel = require('broccoli-funnel');
var cashaySchema = require('broccoli-cashay-schema');
var esTranspiler = require('broccoli-babel-transpiler');
var mergeTrees = require('broccoli-merge-trees');
var path = require('path');
var graphql = require('graphql');
var cashay = require('cashay');

module.exports = {
  name: 'ember-cashay',

  isDevelopingAddon: function() {
    return false;
  },

  included: function(app) {
    // see: https://github.com/ember-cli/ember-cli/issues/3718
    if (typeof(app.import) !== 'function' && app.app) {
      app = app.app;
    }
    this.app = app;

    // Get build config from ember-cli-build.js
    this.addonBuildConfig = app.options['ember-cashay'] || {};
    // Get addon config from the usual config/environment.js
    this.addonConfig = app.project.config(app.env)['ember-cashay'] || {};

    this._super.included.apply(this, arguments);

    if (app.project.pkg['ember-addon'] && !app.project.pkg['ember-addon'].paths) {
      this.schemaDirectory = path.resolve(app.project.root, path.join('tests', 'dummy', 'app', 'graphql'));
    } else {
      this.schemaDirectory = path.resolve(app.project.root, path.join('app', 'graphql'));
    }
  },

  treeForApp: function(appTree) {
    var trees = [ appTree ];

    // Add the server schema (non-prod)
    // TODO: Make the check for `enabled` respect explicit `false`
    if (this.app.env !== 'production' || this.addonConfig['copy-server-schema']) {
      // MAYBE CHANGE TO JUST REMOVE IF PRODUCTION???????
      /*
      var serverTree = new Funnel(this.schemaDirectory, {
        destDir: 'graphql/server'
      });
      trees.push(serverTree);
      */
    }

    // Add the client-safe schema
    var clientTree = cashaySchema(esTranspiler(this.schemaDirectory), {
      cashay: cashay,
      graphql: graphql,
      clientSchemaPath: 'graphql-client/schema.js'
    });
    trees.push(clientTree);

    return mergeTrees(trees);
  }
};
